<?php
/**
 * Visforms decorator class for HTML controls
 *
 * @author       Aicha Vack
 * @package      Joomla.Site
 * @subpackage   com_visforms
 * @link         https://www.vi-solutions.de
 * @license      GNU General Public License version 2 or later; see license.txt
 * @copyright    2012 vi-solutions
 * @since        Joomla 1.6
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Visolutions\Component\Visforms\Site\Event\Visforms\VisformsAfterCustomtextPrepareEvent;
use Joomla\Event\DispatcherInterface;

require_once JPATH_ROOT . '/components/com_visforms/lib/layout/helper/uikit3field.php';
require_once JPATH_ROOT . '/components/com_visforms/lib/layout/helper/bt5field.php';

abstract class VisformsHtmlControl
{
	protected $field;
	protected $layout;
	protected $breakPoints;
	protected $fieldHelper;

	public function __construct($field, $layout) {
		$this->field = $field;
		$this->layout = $layout;
		$this->breakPoints = array('Sm', 'Md', 'Lg', 'Xl');
		$this->setFieldHelper();
	}

    public static function getInstance($field, $layout)
    {
        $fieldtype = $field->getField()->typefield;
        $classname = get_called_class() . ucfirst($layout) . ucfirst($fieldtype);
        if (!class_exists($classname)) {
            // bt5 edit Layout? We have a control implementation for each field type in bt5
            if ($layout === 'editbt5') {
                $classname = get_called_class() . ucfirst('bt5') . ucfirst($fieldtype);
            }
	        // uikit3 edit Layout? We have a control implementation for each field type in uikit3
	        else if ($layout === 'edituikit3') {
		        $classname = get_called_class() . ucfirst('uikit3') . ucfirst($fieldtype);
	        }
	        // edit layout? we have a control implementation für each field type in visforms
            else {
                $classname = get_called_class() . ucfirst('visforms') . ucfirst($fieldtype);
            }
        }

        if (!class_exists($classname)) {
            // fall back on the visform default
            $classname = get_called_class() . ucfirst('visforms') . ucfirst($fieldtype);
        }

        // delegate to the appropriate subclass
        return new $classname($field, $layout);
    }

    abstract public function getControlHtml();

    /**
     * Method to create label html string
     * @return string label html or ''
     */
    public function createLabel()
    {
        return '';
    }

    /**
     * Method to create class attribute value for label tag according to layout
     * @return string class attribute value
     */
    protected function getLabelClass()
    {
        switch ($this->layout) {
	        case 'uikit3' :
	        case 'edituikit3' :
            case 'bt5' :
            case 'editbt5' :
				$labelClass = $this->fieldHelper->getLabelClass();
		        break;
            default :
                $labelClass = ' visCSSlabel ';
                break;
        }
        return $labelClass;
    }

    /**
     * Method to create html for field custom text
     * @return string custom text or ''
     */
    public function getCustomText()
    {
        $class = '';
        $html = '';
    	switch ($this->layout) {
            case 'bt5' :
            case 'editbt5' :
            case 'uikit3' :
            case 'edituikit3' :
			    $class = $this->getCtClasses();
			    break;
		    default :
			    break;
	    }
        $field = $this->field->getField();
        // input
        if (isset($field->customtext) && ($field->customtext != ''))
        {
            PluginHelper::importPlugin('content');
            $customtext = HTMLHelper::_('content.prepare', $field->customtext);
            if (in_array($this->layout, array('uikit3', 'edituikit3'))) {
            	$html .= '<label class="'.$this->getLabelClass().'"></label>';
            }
            $html .= '<div class="'.$class.' ">' . $customtext . '</div>';
        }
        // Trigger onVisformsAfterCustomtextPrepare event to allow changes on field properties before control html is created
        PluginHelper::importPlugin('visforms');
		// make custom adjustments to the custom text html
        $afterCustomTextPrepareEvent = new VisformsAfterCustomtextPrepareEvent('onVisformsAfterCustomtextPrepare', [
            'context' => 'com_visforms.field',
            'layout' => $this->layout,
            'html' => $html
        ]);
        $html = Factory::getContainer()->get(DispatcherInterface::class)->dispatch('onVisformsAfterCustomtextPrepare', $afterCustomTextPrepareEvent)->getArgument('html', $html);
        return $html;
    }

	// Override in control/layoutname/fieldtypename.php for custom classes if necessary
	// proper indentation with regards to label width for elements that do not have a label in front of them, i.e. custom text, error div, checkbox control...
	protected function getCtClasses() {
        if (!empty($this->fieldHelper) && method_exists($this->fieldHelper, 'getCtClasses')) {
            return $this->fieldHelper->getCtClasses();
        }
		$field = $this->field->getField();
		// width always 12 in total
		$ctClasses = ($field->labelBootstrapWidth != "12") ? 'offset-' . $field->labelBootstrapWidth . ' col-' . (12 - $field->labelBootstrapWidth) : 'col-12';
		foreach ($this->breakPoints as $breakPoint) {
			$name = 'labelBootstrapWidth' . $breakPoint;
			$lcBreakPoint = lcfirst($breakPoint);
			$ctClasses .= ($field->$name != "12") ? ' offset-' . $lcBreakPoint . '-' . $field->$name . ' col-' . $lcBreakPoint . '-' . (12 - $field->$name) : '';
		}
		return $ctClasses;
	}

	protected function setFieldHelper() {
    	switch ($this->layout) {
		    case 'uikit3' :
		    case 'edituikit3' :
			    $this->fieldHelper = new VisformsUikit3FieldHelper();
		        $this->fieldHelper->setField($this->field->getField());
			    break;
            case 'bt5' :
            case 'editbt5' :
                $this->fieldHelper = new VisformsBt5FieldHelper();
                $this->fieldHelper->setField($this->field->getField());
                break;
            default:
                break;
	    }
	}
}